import java.util.Random;

/**
 * A class that implements the test for Interval
 * 
 * @author Ana Santos
 * @version 1.0 11/Jul/13 
 **/
public class IntervalRandomTest {

	/**
	 * Interval of MyInteger to be tested.
	 */
	private MyInterval<MyInteger> integerInterval;

	/**
	 * Interval of MyDate to be tested.
	 */
	private MyInterval<MyDate> dateInterval;

	/**
	 * How many tests do we want to perform.
	 */
	private int howMany;

	/**
	 * Max operations.
	 */
	private static int MAX_OPERATIONS = 6;

	/**
	 * Max value of integers created
	 */
	private static int MAX_VALUE = 10;

	/**
	 * Our random object to generate random int.
	 */
	private Random rand;

	/**
	 * Constructor.
	 * 
	 * @param intervalInt - the MyInteger Interval to be tested.
	 * @param intervalDate - the MyDate Interval to be tested.
	 * @param howMany - how many tests we want to perform.
	 */
	public IntervalRandomTest(MyInterval<MyInteger> intervalInt, 
			MyInterval<MyDate> intervalDate, int howMany) {
		integerInterval= intervalInt;
		dateInterval = intervalDate;
		this.howMany = howMany;
		this.rand = new Random();
	}

	/**
	 * Runs the tests.
	 */
	public void run() {

		for (int i = 0; i < howMany; i++) {
			int integerOrDate = rand.nextInt(2);
			
			switch (rand.nextInt(MAX_OPERATIONS)) {
			//max
			case 0: {
				if (integerOrDate == 0)
					System.out.println("max: " + integerInterval.max());
				else 
					System.out.println("max: " + dateInterval.max());
				break;
			}
			//min
			case 1: {
				if (integerOrDate == 0)
					System.out.println("min: " + integerInterval.min());
				else 
					System.out.println("min: " + dateInterval.min());
				break;
			}
			//before
			case 2: {
				if (integerOrDate == 0) {
					MyInterval<MyInteger> other = getNewIntegerInterval();
					System.out.println("The interval " + integerInterval + " is" +
							(integerInterval.isBefore(other)? " " : " not ") + 
							"before the interval " + other);
				} else { 
					MyInterval<MyDate> other = getNewDateInterval();
					System.out.println("The interval " + dateInterval + " is" +
							(dateInterval.isBefore(other)? " " : " not ") + 
							"before the interval " + other);
				} 
				break;
			}
			//elements
			case 3: {
				if (integerOrDate == 0) 
					System.out.println("elements: " + integerInterval.elements());
				else 
					System.out.println("elements: " + dateInterval.elements());
				break;
			}
			//intersects
			case 4: {
				if (integerOrDate == 0) {
					MyInterval<MyInteger> other = getNewIntegerInterval();
					System.out.println("The interval " + integerInterval +
							(integerInterval.intersects(other)? 
									" intersects " : " does not intersect ") + 
									"the interval " + other);
				} else { 
					MyInterval<MyDate> other = getNewDateInterval();
					System.out.println("The interval " + dateInterval +
							(dateInterval.intersects(other)? 
									"intersects " : " does not intersect ") + 
									"the interval " + other);
				} 
				break;
			}
			// the intervals are equal?
			case 5: {
				if (integerOrDate == 0) {
					MyInterval<MyInteger> other = getNewIntegerInterval();
					System.out.println("The intervals " + integerInterval + 
							" and " + other + " are " + 
							(integerInterval.equals(other)? "" : "not ") + "equal ");					
				} else {
					MyInterval<MyDate> other = getNewDateInterval();
					System.out.println("The intervals " + dateInterval + 
							" and " + other + " are " + 
							(dateInterval.equals(other)? "" : "not ") + "equal ");	
				}
				break;
			}
			}
		}
	}

	/**
	 * Returns a random interval of MyInteger
	 * @return an interval of MyInteger
	 */
	private MyInterval<MyInteger> getNewIntegerInterval() {
		MyInteger one = newMyInteger();
		MyInteger two = newMyInteger();
		if (one.isGreaterEq(two)) {
			return new MyInterval<MyInteger>(two, one);
		} else {
			return new MyInterval<MyInteger>(one, two);
		} 
	}

	/**
	 * Returns a random interval of MyDate
	 * @return an interval of MyDate
	 */
	private MyInterval<MyDate> getNewDateInterval() { 

		int month = rand.nextInt(12) + 1;
		int hour = rand.nextInt(24);
		int day = getRandomDay(month);
		MyDate one = new MyDate(hour, day, month);
		month = rand.nextInt(12) + 1;
		hour = rand.nextInt(24);
		day = getRandomDay(month);
		MyDate two = new MyDate(hour, day, month);
		if (one.isGreaterEq(two)) {
			return new MyInterval<MyDate> (two, one);
		} else {
			return new MyInterval<MyDate> (one, two);
		} 
	}

	/**
	 * Get a random day for the given month
	 * @param month - the month
	 * @return the day of the month
	 */
	private int getRandomDay(int month) {
		if (month == 2) {
			return rand.nextInt(28) + 1;
		} else if (month == 6 || month == 4 || month == 9 || month == 11) {
			return rand.nextInt(30) + 1;
		} else {
			return rand.nextInt(31) + 1;
		}
	}
	
	/**
	 * Creates a new random MyInteger.
	 * 
	 * @return a new MyInteger
	 **/
	private MyInteger newMyInteger() {
		return new MyInteger(rand.nextInt(MAX_VALUE));
	}
}