/**
 * Class representing immutable dates with day, month and hour
 * 
 * @author Gloss
 * @version 1.1 17/Jul/2013 as
 */
public class MyDate implements Successorable<MyDate> {

	/**
	 * The month
	 */
	private final int month;

	/**
	 * The day
	 */
	private final int day;

	/**
	 * The hour
	 */
	private final int hour;


	/**
	 * Constructor 
	 * 
	 * @param h - hour
	 * @param d - day
	 * @param m - month
	 */
	public MyDate(int h, int d, int m) {
		hour = h;
		day = d;
		month = m;
	}

	/**
	 * Returns the successor to this date, increasing by one hour
	 * 
	 * @return the next date
	 */
	public MyDate suc() {

		int h;
		int d;
		int m;
		if (isLastMonthDay() && hour == 23) {
			h = 0;
			d = 1; 
			m = month+1;				
		} else if (hour == 23) {
			h = 0;
			d = day + 1;
			m = month;		
		} else {
			h = hour + 1 ;
			d = day; 
			m = month;				
		}
		return new MyDate(h,d,m);
	}

	/**
	 * Verifies if this MyDate is in the last day of the month
	 * 
	 * @return true if the day is the last of the month
	 */
	private boolean isLastMonthDay() {
		boolean result = false;
		switch (month){
		case 1: result = (day == 31); break;
		case 2: result = (day == 28); break;
		case 3: result = (day == 31); break;
		case 4: result = (day == 30); break;
		case 5: result = (day == 31); break;
		case 6: result = (day == 30); break;
		case 7: result = (day == 31); break;
		case 8: result = (day == 31); break;
		case 9: result = (day == 30); break;
		case 10: result = (day == 31); break;
		case 11: result = (day == 30); break;
		case 12: result = (day == 31); break;
		}
		return result;
	}

	/**
	 * Compares this date to another
	 * 
	 * @param other - the MyDate to compare to
	 * @return -1 if this date is less than other
	 * 		    0 if this date is equal to other
	 *          1 if this date is greater than other
	 */
	public int compareTo(MyDate other) {
		if (month < other.month || (month == other.month && day < other.day) || 
				(month == other.month && day == other.day && hour < other.hour))
			return -1;
		else if (month == other.month && day == other.day && hour == other.hour)
			return 0;
		else 
			return 1;
	}

	/**
	 * The hash code for MyDate
	 * 
	 * @return the hash code for this MyDate
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + day;
		result = prime * result + hour;
		result = prime * result + month;
		return result;
	}


	/**
	 * Is this date equal to obj?
	 * 
	 * @param obj - the object to compare
	 * @return true if it's equal, false otherwise
	 */
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!(obj instanceof MyDate))
			return false;
		else {
			MyDate other = (MyDate) obj;
			return  (month==other.month && day==other.day && hour==other.hour);
		}
	}

	/**
	 * Is the given MyDate greater or equal to this?
	 * 
	 * @param e - the element to compare
	 * @return true if this is greater or equal, false otherwise
	 */
	public boolean isGreaterEq(MyDate e) {
		return this.compareTo(e) >= 0; 
	}

	/**
	 * Makes a copy of MyDate
	 * 
	 * @return clone of MyDate
	 */
	public MyDate clone() {
		return this;
	}

	/**
	 * Textual representation of MyDate
	 * 
	 * @return the String representation of MyDate
	 */
	public String toString() {
		return day + "/" + month + " @" + hour + "h";
	}
}