/**
 * Class implementing the Interval specification.
 * Intervals are immutable objects.
 * 
 * @author Gloss
 * @version 1.1 27/Nov/2011 jb
 * 
 * @param <K> Type of the interval
 */

public class MyInterval<K extends Successorable<K>> {
	
	/**
	 * Minimum value
	 */
	private K min;
	
	/**
	 * Maximum value
	 */
	private K max;

	/**
	 * Constructor
	 * 
	 * @param min - min value of the interval
	 * @param max - max value of the interval
	 * 
	 * @requires min <= max
	 */
	public MyInterval(K min, K max){
		this.min = min;
		this.max = max;
	}
	
	/**
	 * Returns the minimum value of the interval
	 * 
	 * @return minimum value
	 */
	public K min() {
		return min;
	}
	
	/**
	 * Returns the maximum value of the interval
	 * 
	 * @return maximum value
	 */
	public K max() {
		return max;
	}
	
	/**
	 * Do the two intervals intersect?
	 * 
	 * @param other - the other interval
	 * @return true if they intersect, false otherwise
	 */
	public boolean intersects(MyInterval<K> other) {
		return (other.max().isGreaterEq(min) && max.isGreaterEq(other.min()))
				|| (min().isGreaterEq(other.min()) && other.max().isGreaterEq(min));
	}

	/**
	 * Is this interval before the given one
	 * i.e. does it end before the other starts?
	 * 
	 * @param other - the other interval
	 * @return true if the other interval minimum is greater or equal
	 * 		   than this interval maximum, false otherwise
	 */
	public boolean isBefore(MyInterval<K> other) {
		return other.min().isGreaterEq(max());
	}

	/**
	 * Returns the elements of this interval
	 * 
	 * @return - set of the elements of the interval
	 */
	public TreeSet<K> elements() {
		K elem = min;
		TreeSet<K> result = new TreeSet<K>();
		while (max.isGreaterEq(elem)){
			result.insert(elem);
			elem = elem.suc();
		}
		return result;
	}
	
	public boolean equals(Object other){
		if (!(other instanceof MyInterval))
			return false;
		@SuppressWarnings("unchecked")
		MyInterval<K> otherInterval = (MyInterval<K>) other;
		return otherInterval.max().equals(max()) && otherInterval.min().equals(min());
	}

	/**
	 * Textual representation of the interval
	 * 
	 * @return String representing the interval
	 */
	public String toString(){
		return "["+min+", "+max+"]";
	}
}